"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WinStateKeeper = exports.AppConfig = void 0;
const fs = require("fs");
const electron_1 = require("electron");
const path_1 = require("path");
const client_1 = require("./client");
const global_1 = require("../../global");
const deepmerge_ts_1 = require("deepmerge-ts");
function isJsonSyntaxCorrect(string) {
    try {
        JSON.parse(string);
    }
    catch {
        return false;
    }
    return true;
}
class AppConfig {
    constructor(path, spaces) {
        this.defaultConfig = {
            hideMenuBar: false,
            disableTray: false,
            devel: false,
            redirectionWarning: true,
            csp: {
                enabled: true,
                thirdparty: {
                    spotify: true,
                    gif: true,
                    hcaptcha: true,
                    youtube: true,
                    twitter: true,
                    twitch: true,
                    streamable: true,
                    vimeo: true,
                    soundcloud: true,
                    paypal: true,
                    audius: true,
                    algolia: true,
                    funimation: true,
                    reddit: true
                }
            },
            blockApi: {
                typingIndicator: false,
                science: true,
            },
            permissions: {
                "video": false,
                "audio": false,
                "fullscreen": true,
                "notifications": false,
                "display-capture": true
            },
            currentInstance: 0,
        };
        this.path = (0, path_1.resolve)(electron_1.app.getPath('userData'), 'config.json');
        if (path !== undefined)
            this.path = path;
        if (spaces !== undefined && spaces > 0)
            this.spaces = spaces;
        else
            this.spaces = 4;
        if (!fs.existsSync(this.path))
            this.write(this.defaultConfig);
        else {
            if (!isJsonSyntaxCorrect(fs.readFileSync(this.path).toString()))
                fs.rmSync(this.path);
            this.write({ ...this.defaultConfig, ...this.get() });
        }
    }
    write(object) {
        fs.writeFileSync(this.path, JSON.stringify(object, null, this.spaces));
    }
    set(object) {
        const oldObject = this.get();
        const newObject = (0, deepmerge_ts_1.deepmerge)(oldObject, object);
        if ((0, global_1.objectsAreSameType)(newObject, oldObject))
            this.write(newObject);
    }
    get() {
        const parsedConfig = JSON.parse(fs.readFileSync(this.path).toString());
        const mergedConfig = (0, deepmerge_ts_1.deepmerge)(this.defaultConfig, parsedConfig);
        if ((0, global_1.objectsAreSameType)(mergedConfig, this.defaultConfig))
            return mergedConfig;
        else
            return this.defaultConfig;
    }
}
exports.AppConfig = AppConfig;
class WinStateKeeper {
    constructor(windowName, file, spaces) {
        this.file = (0, path_1.resolve)(electron_1.app.getPath('userData'), 'windowState.json');
        this.windowName = windowName;
        this.defaultConfig = {
            [this.windowName]: {
                width: client_1.appInfo.minWinWidth + (electron_1.screen.getPrimaryDisplay().workAreaSize.width / 3),
                height: client_1.appInfo.minWinHeight + (electron_1.screen.getPrimaryDisplay().workAreaSize.height / 3),
                isMaximized: false
            }
        };
        if (file !== undefined)
            this.file = file;
        if (spaces !== undefined && spaces > 0)
            this.spaces = spaces;
        else
            this.spaces = 4;
        if (!fs.existsSync(this.file) || this.get().win !== undefined)
            this.write(this.defaultConfig);
        this.initState = {
            width: this.get()[this.windowName].width,
            height: this.get()[this.windowName].height,
            isMaximized: this.get()[this.windowName].isMaximized
        };
    }
    write(object) {
        fs.writeFileSync(this.file, JSON.stringify(object, null, this.spaces));
    }
    set(object) {
        const oldObject = this.get();
        const newObject = (0, deepmerge_ts_1.deepmerge)(oldObject, object);
        if ((0, global_1.objectsAreSameType)(newObject, oldObject))
            this.write(newObject);
    }
    get() {
        const parsedConfig = JSON.parse(fs.readFileSync(this.file).toString());
        if ((0, global_1.objectsAreSameType)(parsedConfig, this.defaultConfig))
            return parsedConfig;
        else
            return this.defaultConfig;
    }
    setState(window) {
        if (window.isMaximized()) {
            this.set({
                [this.windowName]: {
                    width: this.get()[this.windowName].width,
                    height: this.get()[this.windowName].height,
                    isMaximized: true
                }
            });
        }
        else {
            this.set({
                [this.windowName]: {
                    width: window.getNormalBounds().width,
                    height: window.getNormalBounds().height,
                    isMaximized: false
                }
            });
        }
    }
    watchState(window) {
        window.on('resize', () => { this.setState(window); });
        window.on('unmaximize', () => { this.setState(window); });
        window.on('maximize', () => { this.setState(window); });
    }
}
exports.WinStateKeeper = WinStateKeeper;
