"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("../modules/client");
const config_1 = require("../modules/config");
const electron_1 = require("electron");
const getMenu = require("../modules/menu");
const global_1 = require("../../global");
const csp_1 = require("../modules/csp");
const agent_1 = require("../../modules/agent");
const crypto_1 = require("crypto");
const path_1 = require("path");
const safe_1 = require("colors/safe");
const configData = new config_1.AppConfig();
function createMainWindow(startHidden, l10nStrings) {
    let tray;
    const mainWindowState = new config_1.WinStateKeeper('mainWindow');
    const win = new electron_1.BrowserWindow({
        title: electron_1.app.getName(),
        minWidth: client_1.appInfo.minWinWidth,
        minHeight: client_1.appInfo.minWinHeight,
        height: mainWindowState.initState.height,
        width: mainWindowState.initState.width,
        backgroundColor: "#36393F",
        icon: client_1.appInfo.icon,
        show: false,
        webPreferences: {
            preload: electron_1.app.getAppPath() + "/sources/app/renderer/preload/main.js",
            nodeIntegration: false,
            devTools: true,
            defaultFontFamily: {
                standard: 'Arial'
            }
        }
    });
    win.webContents.on('did-fail-load', (_event, errorCode, errorDescription, validatedURL) => {
        if (errorCode <= -100 && errorCode >= -199)
            win.loadFile((0, path_1.resolve)(electron_1.app.getAppPath(), 'sources/assets/web/html/404.html'));
        else if (errorCode === -30) {
            console.warn((0, safe_1.bold)('[WARN]') + ' A page "' + validatedURL + '" was blocked by CSP.');
            return;
        }
        console.error((0, safe_1.bold)('[ERROR]') + ' ' + errorDescription + ' (' + (errorCode * -1) + ')');
        const retry = setInterval(() => {
            if (retry && electron_1.net.isOnline()) {
                clearTimeout(retry);
                win.loadURL(global_1.knownIstancesList[new config_1.AppConfig().get().currentInstance][1].href, { userAgent: (0, agent_1.getUserAgent)(process.versions.chrome) });
            }
        }, 1000);
    });
    win.webContents.once('did-finish-load', () => {
        console.debug("[PAGE] Starting to load the Discord page...");
        if (!startHidden)
            win.show();
        setTimeout(() => win.loadURL(global_1.knownIstancesList[new config_1.AppConfig().get().currentInstance][1].href, { userAgent: (0, agent_1.getUserAgent)(process.versions.chrome) }), 1500);
    });
    if (mainWindowState.initState.isMaximized)
        win.maximize();
    if (configData.get().csp.enabled) {
        win.webContents.session.webRequest.onHeadersReceived((details, callback) => {
            console.debug("[CSP] Overwritting Discord CSP.");
            callback({
                responseHeaders: {
                    ...details.responseHeaders,
                    'Content-Security-Policy': [csp_1.discordContentSecurityPolicy]
                }
            });
        });
    }
    win.webContents.session.webRequest.onBeforeRequest({
        urls: [
            'https://*/api/*/science',
            'https://*/api/*/channels/*/typing',
            'https://*/api/*/track'
        ]
    }, (details, callback) => {
        const configData = (new config_1.AppConfig()).get();
        const cancel = configData.blockApi.science || configData.blockApi.typingIndicator;
        const url = new URL(details.url);
        if (cancel)
            console.debug('[API] Blocking ' + url.pathname);
        if (url.pathname.endsWith('/science') || url.pathname.endsWith('/track'))
            callback({ cancel: configData.blockApi.science });
        else if (url.pathname.endsWith('/typing'))
            callback({ cancel: configData.blockApi.typingIndicator });
        else
            callback({ cancel: false });
    });
    {
        const trustedURLs = [
            global_1.knownIstancesList[new config_1.AppConfig().get().currentInstance][1].origin,
            'devtools://'
        ];
        win.webContents.session.setPermissionCheckHandler((webContents, permission, requestingOrigin) => {
            let websiteURL;
            switch (permission) {
                case "display-capture":
                case "notifications":
                case "media":
                    if (Object.prototype.hasOwnProperty.call(configData.get().permissions, permission))
                        return (configData.get().permissions)[permission];
                    break;
                default:
                    return false;
            }
            (webContents !== null && webContents.getURL() !== "") ? websiteURL = webContents.getURL() : websiteURL = requestingOrigin;
            if (websiteURL === "")
                return false;
            const originURL = new URL(websiteURL).origin;
            for (const secureURL of trustedURLs) {
                if (originURL.startsWith(secureURL)) {
                    return true;
                }
            }
            console.warn(`[${l10nStrings.dialog.common.warning.toLocaleUpperCase()}] ${l10nStrings.dialog.permission.check.denied}`, originURL, permission);
            return false;
        });
        win.webContents.session.setPermissionRequestHandler((webContents, permission, callback, details) => {
            for (const secureURL of trustedURLs) {
                if (new URL(webContents.getURL()).origin !== new URL(secureURL).origin) {
                    return callback(false);
                }
                switch (permission) {
                    case "media": {
                        if (details.mediaTypes === undefined)
                            break;
                        let callbackValue = true;
                        for (const type of details.mediaTypes)
                            callbackValue = callbackValue &&
                                configData.get().permissions[type];
                        return callback(callbackValue);
                    }
                    case "display-capture":
                    case "notifications":
                    case "fullscreen":
                        return callback(configData.get().permissions[permission]);
                    default:
                        return callback(false);
                }
            }
            console.warn('[' + l10nStrings.dialog.common.warning.toLocaleUpperCase() + '] ' + l10nStrings.dialog.permission.request.denied, webContents.getURL(), permission);
            return callback(false);
        });
        win.webContents.session.setDevicePermissionHandler(() => false);
    }
    win.loadFile((0, path_1.resolve)(electron_1.app.getAppPath(), 'sources/assets/web/html/load.html'));
    win.setAutoHideMenuBar(configData.get().hideMenuBar);
    win.setMenuBarVisibility(!configData.get().hideMenuBar);
    {
        let valid = true;
        const spellCheckerLanguages = [electron_1.app.getLocale(), 'en-US'];
        if (electron_1.app.getLocale() === 'en-US')
            valid = false;
        if (valid && process.platform !== 'darwin')
            for (const language in spellCheckerLanguages)
                if (!win.webContents.session.availableSpellCheckerLanguages.includes(language))
                    valid = false;
        if (valid)
            win.webContents.session.setSpellCheckerLanguages(spellCheckerLanguages);
    }
    mainWindowState.watchState(win);
    getMenu.context(win);
    if (!configData.get().disableTray)
        tray = getMenu.tray(win);
    getMenu.bar(global_1.packageJson.repository.url, win);
    let setFavicon;
    win.webContents.once('did-finish-load', () => {
        win.webContents.on('page-favicon-updated', async (_event, favicons) => {
            const t = await tray;
            const faviconRaw = electron_1.nativeImage.createFromDataURL(favicons[0]).toBitmap();
            const faviconHash = (0, crypto_1.createHash)('sha1').update(faviconRaw).digest('hex');
            if (faviconHash === setFavicon)
                return;
            if (new URL(win.webContents.getURL()).origin !== global_1.knownIstancesList[0][1].origin) {
                setFavicon = faviconHash;
                t.setImage(client_1.appInfo.trayIcon);
                win.flashFrame(false);
                return;
            }
            if (!configData.get().disableTray) {
                if (faviconHash === global_1.discordFavicons.default) {
                    t.setImage(client_1.appInfo.trayIcon);
                    win.flashFrame(false);
                }
                else if (faviconHash.startsWith('4')) {
                    t.setImage(client_1.appInfo.trayUnread);
                    win.flashFrame(false);
                }
                else {
                    console.debug("[Mention] Hash: " + faviconHash);
                    t.setImage(client_1.appInfo.trayPing);
                    win.flashFrame(true);
                }
                setFavicon = faviconHash;
            }
        });
    });
    win.on('page-title-updated', (event, title) => {
        if (title.includes("Discord Test Client")) {
            event.preventDefault();
            win.setTitle(electron_1.app.getName() + " (Fosscord)");
        }
        else if (title.includes("Discord")) {
            event.preventDefault();
            win.setTitle(title.replace("Discord", electron_1.app.getName()));
        }
    });
    electron_1.ipcMain.on("cosmetic.load", (event) => {
        win.webContents.on("did-stop-loading", () => {
            console.debug("[IPC] Exposing a 'did-stop-loading' event...");
            event.reply("webContents.did-stop-loading");
        });
    });
    electron_1.ipcMain.on("cosmetic.hideElementByClass", (event, cssRule) => {
        win.webContents.insertCSS(cssRule + ':nth-last-child(2) > *, ' + cssRule + ':nth-last-child(3) > * { display:none; }');
        event.reply("cosmetic.hideElementByClass");
    });
    win.webContents.on('did-finish-load', () => {
        console.debug("[CSS] Injecting a CSS for sidebar animation...");
        win.webContents.insertCSS(".sidebar-2K8pFh{ transition: width .1s cubic-bezier(0.4, 0, 0.2, 1);}");
    });
    electron_1.ipcMain.on('api-exposed', (_event, api) => {
        console.debug("[IPC] Exposing a `getDisplayMedia` and spoffing it as native method.");
        const functionString = `
            navigator.mediaDevices.getDisplayMedia = Function.prototype.call.apply(Function.prototype.bind, [async() => navigator.mediaDevices.getUserMedia(await window['${api}'].desktopCapturerPicker())]);
        `;
        win.webContents.executeJavaScript(functionString + ';0');
    });
    electron_1.ipcMain.on('settings-config-modified', (_event, object) => {
        const config = new config_1.AppConfig();
        if ("hideMenuBar" in object) {
            console.debug("[Settings] Updating menu bar state...");
            win.setAutoHideMenuBar(config.get().hideMenuBar);
            win.setMenuBarVisibility(!config.get().hideMenuBar);
        }
        if ("currentInstance" in object) {
            win.loadURL(global_1.knownIstancesList[config.get().currentInstance][1].href);
        }
    });
    return win;
}
exports.default = createMainWindow;
